﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;

public class GroupByRowCheckBoxCreationFilter
    : IUIElementCreationFilter, 
    IDisposable
{
    #region Private Members
    private UltraGrid grid;
    private string checkBoxColumnKey;
    #endregion // Private Members

    #region Constructor
    public GroupByRowCheckBoxCreationFilter(UltraGrid grid, string checkBoxColumnKey)
    {
        this.grid = grid;
        this.checkBoxColumnKey = checkBoxColumnKey;

        this.grid.InitializeGroupByRow += new InitializeGroupByRowEventHandler(grid_InitializeGroupByRow);
        this.grid.CellChange += new CellEventHandler(grid_CellChange);
    }    

    #endregion // Constructor

    #region Event Handlers

    #region grid_CellChange
    void grid_CellChange(object sender, CellEventArgs e)
    {
        // Is the cell in the checkbox column?
        if (string.Compare(e.Cell.Column.Key, this.checkBoxColumnKey) == 0)
        {
            // Walk up the parent chain and re-fire InitializeRow for
            // each parent row up the chain. This is to re-synchronize the parent
            // GroubyRows. 
            UltraGridRow row = e.Cell.Row.ParentRow;
            while(row != null)
            {
                row.Refresh(RefreshRow.FireInitializeRow, false);
                row = row.ParentRow;
            }
        }
    }
    #endregion // grid_CellChange

    #region grid_InitializeGroupByRow
    void grid_InitializeGroupByRow(object sender, InitializeGroupByRowEventArgs e)
    {
        // Figure out when the checked state of the checkbox in the GroupByRow should
        // be and assign it to the Tag property of the GroupByRow. 
        object newTag = GetCurrentCheckState(e.Row);

        // If the new tag is the same as the existing one, then we don't need to do anything else.
        if (object.Equals(e.Row.Tag, newTag))
            return;

        // Apply the new tag to the GroupbyRow.
        e.Row.Tag = newTag; 

        // We need to dirty the UIElements of the GroupByRow so that it's display 
        // updates and the CheckBoxUIElement's CheckState gets set based on the new tag.
        UIElement rowElement = e.Row.GetUIElement();
        if (rowElement != null)
            rowElement.DirtyChildElements();
    }
    #endregion // grid_InitializeGroupByRow 

    #endregion // Event Handlers 

    #region GetCurrentCheckState
    private CheckState GetCurrentCheckState(UltraGridGroupByRow groupByRow)
    {
        // Figure out the CheckState of the CheckBox in this GroupByRow by looping through
        // all of the child rows recusively. 
        CheckState? checkState = null;
        return GetCurrentCheckState(groupByRow.Rows, checkState);
    }

    private CheckState GetCurrentCheckState(RowsCollection rows, CheckState? aggregateCheckState)
    {
        foreach (UltraGridRow row in rows)
        {
            CheckState checkState;
            if (row.IsGroupByRow)
                checkState = GetCurrentCheckState((UltraGridGroupByRow)row);
            else
                checkState = GetCurrentCheckState(row);

            if (aggregateCheckState == null)
            {
                // If the aggregateCheckState has not yet been set, set it to the CheckState of the 
                // first row we encounter.
                aggregateCheckState = checkState;
            }
            else if (checkState != aggregateCheckState.Value)
            {
                // If the aggregateCheckState has a value, and we hit a row with a different
                // value, then return the Indeterinate state, becuase we have mixed checked and
                // unchecked rows in this collection. 
                return CheckState.Indeterminate;
            }
        }

        // If we get here, it means that all of the rows in the collection have the 
        // same CheckState. So just return that CheckState. 
        return aggregateCheckState.Value;
    }

    private CheckState GetCurrentCheckState(UltraGridRow row)
    {
        // Determine the CheckState of a single data row. 

        UltraGridCell cell = row.Cells[this.checkBoxColumnKey];

        // Use the Text property of the cell, not the Value. Value will not be correct
        // for any cell that is currently in edit mode. Text wil work for cells both in and
        // out of edit mode. 
        string text = row.Cells[this.checkBoxColumnKey].Text;

        UltraGridColumn column = cell.Column;
        Type dataType = column.DataType;

        // If this is a CheckState column, try to convert the text into a CheckState. 
        if (dataType.IsAssignableFrom(typeof(CheckState)))
            return (CheckState)Enum.Parse(dataType, text);

        // If it's a boolean column, convert the text to a bool, then the bool into a
        // CheckState. 
        if (dataType.IsAssignableFrom(typeof(bool)))
        {
            bool bValue = bool.Parse(text);
            return bValue
                ? CheckState.Checked
                : CheckState.Unchecked;
        }

        // If we get here, it means the column is neither a CheckState nor a bool 
        // column. So we don't know how to handle that. 
        Debug.Fail("Could not convert the cell value into a CheckState");
        return CheckState.Unchecked;
    }
    #endregion // GetCurrentCheckState

    #region IUIElementCreationFilter Members

    void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
    {
        if (parent is GroupByRowUIElement)
        {
            // Get the groupByRow, and if we can't find one, bail out. 
            UltraGridGroupByRow row = parent.GetContext(typeof(UltraGridGroupByRow)) as UltraGridGroupByRow;
            if (row == null)
                return;

            // Find the groupByRowDescriptionUIElement. If we can't find one, bail out.             
            UIElement groupByRowDescriptionUIElement = parent.GetDescendant(typeof(GroupByRowDescriptionUIElement));
            if (groupByRowDescriptionUIElement == null)
                return;

            // If there is an existing GroupByRowCheckBoxUIElement, re-use it; otherwise, create
            // a new one. 
            GroupByRowCheckBoxUIElement groupByRowCheckBoxUIElement = parent.GetDescendant(typeof(GroupByRowCheckBoxUIElement)) as GroupByRowCheckBoxUIElement;
            if (groupByRowCheckBoxUIElement == null)
            {
                groupByRowCheckBoxUIElement = new GroupByRowCheckBoxUIElement(parent, this.checkBoxColumnKey);
                parent.ChildElements.Add(groupByRowCheckBoxUIElement);
            }

            // Get the state of the GroupByRowCheckBoxUIElement from the Tag property of the
            // GroupByRow. 
            CheckState checkState = row.Tag == null
                ? CheckState.Unchecked
                : (CheckState)row.Tag;

            groupByRowCheckBoxUIElement.CheckState = checkState;

            // Position the groupByRowCheckBoxUIElement to the location of the groupByRowDescriptionUIElement
            // and just change it's size to the normal CheckBox size (13 x 13)
            groupByRowCheckBoxUIElement.Rect = new Rectangle(
                groupByRowDescriptionUIElement.Rect.X,
                groupByRowDescriptionUIElement.Rect.Y,
                13,
                13);

            // Shift the groupByRowDescriptionUIElement by 15 to make room for the 
            // groupByRowCheckBoxUIElement.
            groupByRowDescriptionUIElement.Rect = new Rectangle(
                groupByRowDescriptionUIElement.Rect.X + 15,
                groupByRowDescriptionUIElement.Rect.Y,
                groupByRowDescriptionUIElement.Rect.Width - 15,
                groupByRowDescriptionUIElement.Rect.Height);
        }
    }

    bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
    {
        // do nothing
        return false;
    }

    #endregion

    #region IDisposable Members

    void IDisposable.Dispose()
    {
        this.grid.InitializeGroupByRow -= new InitializeGroupByRowEventHandler(grid_InitializeGroupByRow);
        this.grid.CellChange -= new CellEventHandler(grid_CellChange);
    }

    #endregion

    #region GroupByRowCheckBoxUIElement class
    public class GroupByRowCheckBoxUIElement : CheckBoxUIElement
    {
        // The column in the grid whose checkbox will by synched with the GroupByRows. 
        private string checkBoxColumnKey;

        public GroupByRowCheckBoxUIElement(UIElement parent, string checkBoxColumnKey)
            : base(parent)
        {
            this.checkBoxColumnKey = checkBoxColumnKey;
        }

        protected override void OnCheckStateChange()
        {
            base.OnCheckStateChange();

            // When the check state of the CheckBox changes, we need to update the Tag property
            // on the GroupByRow. 
            UltraGridGroupByRow groupByRow = this.GetContext(typeof(UltraGridGroupByRow)) as UltraGridGroupByRow;
            SetCheckState(groupByRow);
        }

        private void SetCheckState(UltraGridGroupByRow groupByRow)
        {
            // Set the CheckState on the GroupByRow.
            groupByRow.Tag = this.CheckState;

            // Set the same CheckState on all of it's child rows, recursively. 
            SetCheckState(groupByRow.Rows);

            // If the GroupByRow has a UIElement, dirty it's child elements so that the
            // display is updated. This is not neccessary for the GroupByRow that the user clicked
            // on, and it's not neccessary for any data rows under it. But it is neccessary for any 
            // of it's children which are GroupByRows. 
            UIElement element = groupByRow.GetUIElement();
            if (element != null)
                element.DirtyChildElements();
        }

        private void SetCheckState(RowsCollection rows)
        {
            foreach (UltraGridRow row in rows)
            {
                if (row is UltraGridGroupByRow)
                    SetCheckState((UltraGridGroupByRow)row);
                else
                    SetCheckState(row);
            }
        }

        private void SetCheckState(UltraGridRow row)
        {            
            UltraGridColumn column = row.Cells[this.checkBoxColumnKey].Column;
            Type dataType = column.DataType;

            // If the column is a CheckState column, then set the value of the cell 
            // directly to the new CheckState. 
            if (dataType.IsAssignableFrom(typeof(CheckState)))
                row.Cells[this.checkBoxColumnKey].Value = this.CheckState;

            // If the colun is a boolean column, we need to convert the CheckState value
            // into a bool.
            if (dataType.IsAssignableFrom(typeof(bool)))
                row.Cells[this.checkBoxColumnKey].Value = this.CheckState == CheckState.Checked ? true : false;

            // This line of code is optional. 
            // If we do not call Update on the row, then it will show a pencil icon indicating
            // that there are still pending changes.
            row.Update();
        }
    }
    #endregion // GroupByRowCheckBoxUIElement class
    
}

    
